// Surdu Cristina 331CA BLACKJACK

#include "pcd8544.h"

//Performs IO & LCD controller initialization
void lcd_init(void)
{
    // Pull-up on reset pin
    LCD_PORT |= LCD_RST_PIN;
	
	// Set output bits on lcd port
	LCD_DDR |= LCD_RST_PIN | LCD_CE_PIN | LCD_DC_PIN | LCD_DATA_PIN | LCD_CLK_PIN;
    
	// Wait after VCC high for reset (max 30ms)
    _delay_ms(15);
	//_delay_ms(20);
    
    // Toggle display reset pin
    LCD_PORT &= ~LCD_RST_PIN;
    lcd_delay();
    LCD_PORT |= LCD_RST_PIN;

    // Disable LCD controller
    LCD_PORT |= LCD_CE_PIN;

    lcd_send(0x21, LCD_CMD);  // LCD Extended Commands
    lcd_send(0xC8, LCD_CMD);  // Set LCD Vop(Contrast)
    lcd_send(0x06, LCD_CMD);  // Set Temp coefficent
    lcd_send(0x13, LCD_CMD);  // LCD bias mode 1:48
    lcd_send(0x20, LCD_CMD);  // Standard Commands, Horizontal addressing
    lcd_send(0x0C, LCD_CMD);  // LCD in normal mode
    
    // Clear lcd
    lcd_clear();
	
	// For using printf
	fdevopen((int *)lcd_chr, 0);
}

// Set display contrast. Note: No change is visible at ambient temperature
void lcd_contrast(unsigned char contrast)
{
	lcd_send(0x21, LCD_CMD);				// LCD Extended Commands
    lcd_send(0x80 | contrast, LCD_CMD);		// Set LCD Vop(Contrast)
    lcd_send(0x20, LCD_CMD);				// LCD std cmds, hori addr mode
}

// Clears the display
void lcd_clear(void)
{
	int i, j;
	lcdCacheIdx = 0;
	
	lcd_base_addr(lcdCacheIdx);
	
    // Set the entire cache to zero and write 0s to lcd
	for( i=0;i < LCD_CACHE_SIZE * 8;i++) {
		lcd_send(0, LCD_DATA);
    }
	
	for(i = 0; i < 6; i++)
		for(j = 0; j < 84; j++)
		{
			buf[i][j] = 0x00;
		}
}

// Sets cursor location to xy location corresponding to basic font size 
void lcd_goto_xy(unsigned char x, unsigned char y)
{
    lcdCacheIdx = (x-1)*6 + (y-1)*84;
}

// Displays a character at current cursor location 
void lcd_chr(char chr)
{
	unsigned char i;
	lcd_base_addr(lcdCacheIdx);

    // 5 pixel wide characters and add space
	for(i=0;i<5;i++) {
		lcd_send(pgm_read_byte(&font5x7[chr-32][i]) << 1, LCD_DATA);
    }
	lcd_send(0, LCD_DATA);
	
	lcdCacheIdx += 6;
}

// Displays string at current cursor location and increment cursor location 
void lcd_str(char *str)
{
    while(*str) {
        lcd_chr(*str++);
    }
}

// Set the base address of the lcd
void lcd_base_addr(unsigned int addr) {
	lcd_send(0x80 |(addr % LCD_X_RES), LCD_CMD);
	lcd_send(0x40 |(addr / LCD_X_RES), LCD_CMD);
}

// Sends data to display controller
void lcd_send(unsigned char data, LcdCmdData cd)
{

	unsigned char i;
	// Data/DC are outputs for the lcd (all low)
	LCD_DDR |= LCD_DATA_PIN | LCD_DC_PIN;
	
    // Enable display controller (active low)
    LCD_PORT &= ~LCD_CE_PIN;

    // Either command or data
    if(cd == LCD_DATA) {
        LCD_PORT |= LCD_DC_PIN;
    } else {
        LCD_PORT &= ~LCD_DC_PIN;
    }
	
	for(i=0;i<8;i++) {
	
		// Set the DATA pin value
		if((data>>(7-i)) & 0x01) {
			LCD_PORT |= LCD_DATA_PIN;
		} else {
			LCD_PORT &= ~LCD_DATA_PIN;
		}
		
		// Toggle the clock
		LCD_PORT |= LCD_CLK_PIN;
		LCD_PORT &= ~LCD_CLK_PIN;
	}

	// Disable display controller
    LCD_PORT |= LCD_CE_PIN;
	
	// Data/DC can be used as button inputs when not sending to LCD (/w pullups)
	LCD_DDR &= ~(LCD_DATA_PIN | LCD_DC_PIN);
	LCD_PORT |= LCD_DATA_PIN | LCD_DC_PIN;
}

void LCD_goto(unsigned char x, unsigned char y)
{
	lcd_send(0x80 | x,  LCD_CMD);   //column
	lcd_send(0x40 | y, LCD_CMD);   //row

	cursor_row = y;
	cursor_col = x;
}


void LCD_setpixel(unsigned char x, unsigned char y)
{
	unsigned char value;
	unsigned char row;
	
	row = y / 8;

	value = buf[row][x];
	value |= (1 << (y % 8));
	buf[row][x] = value;

	LCD_goto(x, row);
	lcd_send(value, LCD_DATA);
}

void initButtons() {
	PORTA = 0;
	DDRA  = 0;
	PORTA |= RESET;
	PORTA |= YES;
	PORTA |= NO;
}

//Check if button was pressed.
void GetButton(unsigned char *apasa) {

	if ((PINA & YES) == 0 && !bY) {
		*apasa = Y;
		bY = 1;
		bN = 0;
		bR = 0;
		return;
	}
	
	if ((PINA & NO) == 0 && !bN) {
		*apasa = N;
		bY = 0;
		bN = 1;
		bR = 0;
		return;
	}
	if ((PINA & RESET) == 0 &&  !bR) {
		*apasa = R;
		bY = 0;
		bN = 0;
		bR = 1;
		return;
	}
}

void blackZero ()
{
	int i, j;
	for (i = 0; i < 9; i++)
		for (j = 0; j < 12; j++)
			carte[i][j] = 0;
}

void blackInit (unsigned char val)
{
	blackZero();
	int x, y;
	for (x = 0; x < 9; x++)
	{
		carte[x][0] = 1;
		carte[x][11] = 1;
	}
	for (y = 0; y < 12; y++)
	{
		carte[0][y] = 1;
		carte[8][y] = 1;
	}
	
	switch (val)
	{
	//prima carte
	case 1: 
		for (y = 2; y < 10; y++)
			carte[4][y] = 1;
		break;
	//a doua
	case 2:
		for (y = 2; y < 5; y++)
			carte[2][y] = 1;
		for (y = 5; y < 10; y++)
			carte[6][y] = 1;
		for (x = 2; x < 7; x++) 
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		break;
	//a treia carte
	case 3:
		for (y = 2; y < 10; y++)
			carte[6][y] = 1;
		for (x = 2; x < 7; x++)
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		break;
	//a patra carte
	case 4:
		for (x = 2; x < 7; x++)
			carte[x][5] = 1;
		for (y = 2; y < 6; y++)
			carte[6][y] = 1;
		for (y = 5; y < 10; y++)
			carte[2][y] = 1;
		break;
	//a cincea carte
	case 5:
		for (x = 2; x < 7; x++)
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		for (y = 2; y < 6; y++)
			carte[6][y] = 1;
		for (y = 5; y < 10; y++)
			carte[2][y] = 1;
		break;
	//a sasea carte
	case 6:
		for (y = 2; y < 10; y++)
			carte[2][y] = 1;
		for (y = 2; y < 6; y++)
			carte[6][y] = 1;
		for (x = 2; x < 7; x++)
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		break;
	//a saptea carte
	case 7:
		for (x = 2; x < 7; x++)
			carte[x][9] = 1;
		for (y = 2; y < 10; y++)
			carte[6][y] = 1;
		break;
	//a opta carte
	case 8:
		for (x = 2; x < 7; x++)
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		for (y = 2; y < 10; y++)
		{
			carte[2][y] = 1;
			carte[6][y] = 1;
		}
		break;
	//a noua carte
	case 9:
		for (y = 2; y < 10; y++)
			carte[6][y] = 1;
		for (y = 5; y < 10; y++)
			carte[2][y] = 1;
		for (x = 2; x < 7; x++)
		{
			carte[x][2] = 1;
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		break;
	//a zecea carte
	case 10:
		for (x = 2; x < 9; x++)
			carte[x][6] = 1;
		for (y = 2; y < 10; y++)
			carte[4][y] = 1;
		break;
	//a unspea carte
	case 11:
		for (x = 2; x < 7; x++)
		{
			carte[x][5] = 1;
			carte[x][9] = 1;
		}
		for (y = 2; y < 10; y++)
		{
			carte[2][y] = 1;
			carte[6][y] = 1;
		}
		break;
	//carte dreapta sus (?)
	case 12:
		carte[4][2] = 1;
		for (x = 2; x < 5; x++)
			carte[x][9] = 1;
		for (y = 4; y < 10; y++)
			carte[4][y] = 1;
		break;
	}
}

void blackDraw (unsigned char val)
{
	blackInit (val);
	int x, y;
	for (x = 0; x < 9; x++)
		for (y = 0; y < 12; y++)
			if (carte[x][y] == 1)
				LCD_setpixel(x+lin, col-y);
	lin += 9;
}

void blackDrawWhenever (unsigned char val)
{	
	blackInit (val);
	int x, y;
	for (x = 0; x < 9; x++)
		for (y = 0; y < 12; y++)
			if (carte[x][y] == 1)
				LCD_setpixel(x+70, 14-y);
}

void myRand ()
{	int nr_carte = (rand() / (RAND_MAX / (rand()) + 1)) % 11 + 1;
	while (carte_globala == nr_carte)
		nr_carte = (rand() / (RAND_MAX / (rand()) + 1)) % 11 + 1;
	carte_globala = nr_carte;
}

void reset(){
	lcd_clear();
	blackZero();
	pushed = 3;
	lcd_clear();
	score = 0;
	bY = 0;
	bN = 0;
	bR = 0;
	lin = 4;

	blackZero();	
	blackDrawWhenever(12);
	
	carte_globala = (rand() / (RAND_MAX / (rand()) + 1)) % 11 + 1;
	score += carte_globala;
	blackDraw(carte_globala);
	
	myRand();
	score += carte_globala;
	blackDraw(carte_globala);

}

//main -- Program starts here
int main()
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	initButtons();
	lcd_clear();
	
	reset();

	while(1)
	{
		pushed = 3;
		lcd_goto_xy(1, 1);
		sprintf(message, "Score:%d", score);
		lcd_str(message);
		
		if (score == 21)
		{
			lcd_goto_xy(1, 1);
			sprintf(message, "Score:%d", score);
			lcd_str(message);
			lcd_goto_xy(1, 2);
			lcd_str("Blackjack");
			_delay_ms(200);
			reset();
		}
		
		GetButton(&pushed);
			
		if(pushed == Y)
		{
			myRand();
			blackDraw(carte_globala);
			score += carte_globala;
			if (score > 21)
			{
				lcd_goto_xy(1, 1);
				sprintf(message, "Score:%d", score);
				lcd_str(message);
				lcd_goto_xy(1, 2);
				lcd_str("Ai pierdut");
				_delay_ms(200);
				lcd_clear();
				reset();
			}

			if (score >= 18 && score <= 21)
			{
				lcd_goto_xy(1, 1);
				sprintf(message, "Score:%d", score);
				lcd_str(message);
				lcd_goto_xy(1, 2);
				lcd_str("Ai castigat");
				_delay_ms(200);
				lcd_clear();
				reset();
			}

			_delay_ms(100);
		}
		
		if(pushed == N)
		{
			if (score < 18)
			{
				lcd_goto_xy(1, 1);
				sprintf(message, "Score:%d", score);
				lcd_str(message);
				lcd_goto_xy(1, 2);
				lcd_str("Ai pierdut");
				_delay_ms(200);
				lcd_clear();
				reset();
			}
			if (score >= 18 && score <= 21)
			{
				lcd_goto_xy(1, 1);
				sprintf(message, "Score:%d", score);
				lcd_str(message);
				lcd_goto_xy(1, 2);
				lcd_str("Ai castigat");
				_delay_ms(200);
				lcd_clear();
				reset();
			}
		}
		if(pushed == R)
		{
			lcd_clear();
			reset();
		}
		bY = 0;
		bN = 0;
		bR = 0;
		_delay_ms(2);
	}
	return 0;
}